@@ -230,6 +230,7 @@ typedef struct
 }			POOL_HASH_HEADER;
 
 extern int	pool_hash_init(int nelements);
+extern size_t pool_hash_size(int nelements);
 extern POOL_CACHEID * pool_hash_search(POOL_QUERY_HASH * key);
 extern int	pool_hash_delete(POOL_QUERY_HASH * key);
 extern uint32 hash_any(unsigned char *k, int keylen);
@@ -242,8 +243,8 @@ extern int pool_fetch_cache(POOL_CONNECTION_POOL * backend, const char *query, c
 extern int pool_catalog_commit_cache(POOL_CONNECTION_POOL * backend, char *query, char *data, size_t datalen);
 
 extern bool pool_is_likely_select(char *query);
-extern bool pool_is_table_in_black_list(const char *table_name);
-extern bool pool_is_table_in_white_list(const char *table_name);
+extern bool pool_is_table_in_unsafe_list(const char *table_name);
+extern bool pool_is_table_in_safe_list(const char *table_name);
 extern bool pool_is_allow_to_cache(Node *node, char *query);
 extern int	pool_extract_table_oids(Node *node, int **oidsp);
 extern int	pool_extract_withclause_oids(Node *with, int *oidsp);
@@ -278,9 +279,12 @@ extern POOL_SHMEM_STATS * pool_get_shmem_storage_stats(void);
 extern POOL_TEMP_QUERY_CACHE * pool_get_current_cache(void);
 extern POOL_TEMP_QUERY_CACHE * pool_get_current_cache(void);
 extern void pool_discard_temp_query_cache(POOL_TEMP_QUERY_CACHE * temp_cache);
+extern void pool_discard_current_temp_query_cache(void);
 
 extern void pool_shmem_lock(void);
 extern void pool_shmem_unlock(void);
 extern bool pool_is_shmem_lock(void);
 
+extern void InvalidateQueryCache(int tableoid, int dboid);
+
 #endif							/* POOL_MEMQCACHE_H */
